@@ -5,14 +5,14 @@ import time
 import warnings
 from email import charset
 
-__version__ = "4.4.4"
+__version__ = "4.6.0"
 
 os.environ['TZ'] = 'UTC'
 if hasattr(time, 'tzset'):
     time.tzset()
 
 if time.tzname[0] != 'UTC':
-    warnings.warn('Timezone must be set to UTC instead of %s', time.tzname[0])
+    warnings.warn('Timezone must be set to UTC instead of %s' % time.tzname[0])
 
 # set email encoding for utf-8 to 'quoted-printable'
 charset.add_charset('utf-8', charset.QP, charset.QP)
